@@ -27,19 +27,6 @@
 
 G_BEGIN_DECLS
 
-/**
- * GpkFreqEnum:
- *
- * The frequency type
- **/
-typedef enum {
-	GPK_FREQ_ENUM_HOURLY,
-	GPK_FREQ_ENUM_DAILY,
-	GPK_FREQ_ENUM_WEEKLY,
-	GPK_FREQ_ENUM_NEVER,
-	GPK_FREQ_ENUM_UNKNOWN
-} GpkFreqEnum;
-
 /**
  * GpkUpdateEnum:
  *
@@ -94,13 +81,46 @@ typedef guint PkMediaTypeEnum;
 #define PK_ERROR_ENUM_NOT_AUTHORIZED		(PK_ERROR_ENUM_MEDIA_CHANGE_REQUIRED + 1)
 #endif
 
+/* constants defined in 0.4.9 */
+#if (!PK_CHECK_VERSION(0,4,9))
+#define PK_ERROR_ENUM_UPDATE_NOT_FOUND			(PK_ERROR_ENUM_NOT_AUTHORIZED + 1)
+#define PK_ERROR_ENUM_CANNOT_INSTALL_REPO_UNSIGNED	(PK_ERROR_ENUM_UPDATE_NOT_FOUND + 1)
+#define PK_ERROR_ENUM_CANNOT_UPDATE_REPO_UNSIGNED	(PK_ERROR_ENUM_CANNOT_INSTALL_REPO_UNSIGNED + 1)
+#define PK_STATUS_ENUM_WAITING_FOR_AUTH			(PK_STATUS_ENUM_WAITING_FOR_LOCK + 1)
+#undef PK_STATUS_ENUM_UNKNOWN
+#define PK_STATUS_ENUM_UNKNOWN				(PK_STATUS_ENUM_WAITING_FOR_AUTH + 1)
+#endif
+
+/* constants defined in 0.5.0 */
+#if (!PK_CHECK_VERSION(0,5,0))
+#define PK_EXIT_ENUM_NEED_UNTRUSTED			(PK_EXIT_ENUM_MEDIA_CHANGE_REQUIRED + 1)
+#define PK_ERROR_ENUM_CANNOT_GET_FILELIST		(PK_ERROR_ENUM_CANNOT_UPDATE_REPO_UNSIGNED + 1)
+#define PK_ERROR_ENUM_CANNOT_GET_REQUIRES		(PK_ERROR_ENUM_CANNOT_GET_FILELIST + 1)
+#define PK_ERROR_ENUM_CANNOT_DISABLE_REPOSITORY		(PK_ERROR_ENUM_CANNOT_GET_REQUIRES + 1)
+#endif
+
+/* functions defined in 0.5.0 */
+#if (!PK_CHECK_VERSION(0,5,0))
+#define pk_error_code_is_need_untrusted			gpk_error_code_is_need_untrusted
+#define pk_client_set_only_trusted(c,t)
+#endif
+
+/* constants defined in 0.5.1 */
+#if (!PK_CHECK_VERSION(0,5,1))
+#define PK_RESTART_ENUM_SECURITY_SESSION		(PK_RESTART_ENUM_SYSTEM + 1)
+#define PK_RESTART_ENUM_SECURITY_SYSTEM			(PK_RESTART_ENUM_SECURITY_SESSION + 1)
+#define PK_STATUS_ENUM_SCAN_PROCESS_LIST		(PK_STATUS_ENUM_WAITING_FOR_AUTH + 1)
+#define PK_STATUS_ENUM_CHECK_EXECUTABLE_FILES		(PK_STATUS_ENUM_SCAN_PROCESS_LIST + 1)
+#define PK_STATUS_ENUM_CHECK_LIBRARIES			(PK_STATUS_ENUM_CHECK_EXECUTABLE_FILES + 1)
+#undef PK_STATUS_ENUM_UNKNOWN
+#define PK_STATUS_ENUM_UNKNOWN				(PK_STATUS_ENUM_CHECK_LIBRARIES + 1)
+#endif
+
 void		 gpk_enum_test				(gpointer	 data);
 const gchar	*gpk_role_enum_to_localised_past	(PkRoleEnum	 role)
 							 G_GNUC_CONST;
 const gchar	*gpk_role_enum_to_localised_present	(PkRoleEnum	 role)
 							 G_GNUC_CONST;
-GpkFreqEnum	 gpk_freq_enum_from_text		(const gchar	*freq);
-const gchar	*gpk_freq_enum_to_text			(GpkFreqEnum	 freq);
 GpkUpdateEnum	 gpk_update_enum_from_text		(const gchar	*update);
 const gchar	*gpk_update_enum_to_text		(GpkUpdateEnum	 update);
 const gchar	*gpk_role_enum_to_icon_name		(PkRoleEnum	 role);
