@@ -21,6 +21,7 @@
 
 #include "config.h"
 
+#include <locale.h>
 #include <glib.h>
 #include <glib/gi18n.h>
 
@@ -51,6 +52,7 @@
 #include "gpk-enum.h"
 #include "gpk-helper-repo-signature.h"
 #include "gpk-helper-eula.h"
+#include "gpk-helper-deps-update.h"
 
 #define GPK_UPDATE_VIEWER_AUTO_QUIT_TIMEOUT	10 /* seconds */
 #define GPK_UPDATE_VIEWER_AUTO_RESTART_TIMEOUT	60 /* seconds */
@@ -70,11 +72,13 @@ static PkControl *control = NULL;
 static PkPackageList *update_list = NULL;
 static GpkHelperRepoSignature *helper_repo_signature = NULL;
 static GpkHelperEula *helper_eula = NULL;
+static GpkHelperDepsUpdate *helper_deps_update = NULL;
 static EggMarkdown *markdown = NULL;
 static PkPackageId *package_id_last = NULL;
 static PkRestartEnum restart_update = PK_RESTART_ENUM_NONE;
 static guint size_total = 0;
 static GConfClient *gconf_client = NULL;
+static gchar **install_package_ids = NULL;
 
 enum {
 	GPK_UPDATES_COLUMN_TEXT,
@@ -85,6 +89,7 @@ enum {
 	GPK_UPDATES_COLUMN_CLICKABLE,
 	GPK_UPDATES_COLUMN_RESTART,
 	GPK_UPDATES_COLUMN_SIZE,
+	GPK_UPDATES_COLUMN_SIZE_DISPLAY,
 	GPK_UPDATES_COLUMN_PERCENTAGE,
 	GPK_UPDATES_COLUMN_STATUS,
 	GPK_UPDATES_COLUMN_DETAILS_OBJ,
@@ -347,12 +352,12 @@ gpk_update_viewer_button_install_cb (GtkWidget *widget, gpointer data)
 	gboolean ret;
 	gboolean valid;
 	gboolean update;
-	gboolean selected_all = TRUE;
 	gboolean selected_any = FALSE;
 	gchar *package_id;
 	GError *error = NULL;
 	GPtrArray *array = NULL;
 	gchar **package_ids = NULL;
+	PkInfoEnum info;
 
 	/* check connection */
 	ret = gpk_update_viewer_button_check_connection (size_total);
@@ -374,7 +379,9 @@ gpk_update_viewer_button_install_cb (GtkWidget *widget, gpointer data)
 
 	/* find out how many we should update */
 	while (valid) {
-		gtk_tree_model_get (model, &iter, GPK_UPDATES_COLUMN_SELECT, &update,
+		gtk_tree_model_get (model, &iter,
+				    GPK_UPDATES_COLUMN_INFO, &info,
+				    GPK_UPDATES_COLUMN_SELECT, &update,
 				    GPK_UPDATES_COLUMN_ID, &package_id, -1);
 
 		/* set all the checkboxes insensitive */
@@ -382,14 +389,12 @@ gpk_update_viewer_button_install_cb (GtkWidget *widget, gpointer data)
 				    GPK_UPDATES_COLUMN_CLICKABLE, FALSE,
 				    GPK_UPDATES_COLUMN_SENSITIVE, FALSE, -1);
 
-		/* ay selected? */
-		if (!update)
-			selected_all = FALSE;
-		else
+		/* any selected? */
+		if (update)
 			selected_any = TRUE;
 
-		/* do something with the data */
-		if (update) {
+		/* if selected, and not added previously because of deps */
+		if (update && info != PK_INFO_ENUM_AVAILABLE) {
 			g_ptr_array_add (array, package_id);
 		} else {
 			/* need to free the one in the array later */
@@ -424,11 +429,15 @@ gpk_update_viewer_button_install_cb (GtkWidget *widget, gpointer data)
 		goto out;
 	}
 
-	/* set correct view */
+	/* save for finished */
 	package_ids = pk_package_ids_from_array (array);
-	ret = pk_client_update_packages (client_primary, package_ids, &error);
+	g_strfreev (install_package_ids);
+	install_package_ids = g_strdupv (package_ids);
+
+	/* get packages that also have to be updated */
+	ret = pk_client_get_depends (client_primary, pk_bitfield_value (PK_FILTER_ENUM_NOT_INSTALLED), package_ids, TRUE, &error);
 	if (!ret) {
-		egg_warning ("cannot update packages: %s", error->message);
+		egg_warning ("cannot get depends for updates: %s", error->message);
 		g_error_free (error);
 		goto out;
 	}
@@ -529,7 +538,9 @@ gpk_update_viewer_details_cb (PkClient *client, const PkDetailsObj *obj, gpointe
 	gtk_tree_path_free (path);
 	gtk_list_store_set (list_store_updates, &iter,
 			    GPK_UPDATES_COLUMN_DETAILS_OBJ, (gpointer) pk_details_obj_copy (obj),
-			    GPK_UPDATES_COLUMN_SIZE, (gint)obj->size, -1);
+			    GPK_UPDATES_COLUMN_SIZE, (gint)obj->size,
+			    GPK_UPDATES_COLUMN_SIZE_DISPLAY, (gint)obj->size,
+			    -1);
 	/* in cache */
 	if (obj->size == 0)
 		gtk_list_store_set (list_store_updates, &iter,
@@ -570,9 +581,11 @@ gpk_update_viewer_package_cb (PkClient *client, const PkPackageObj *obj, gpointe
 	GtkTreeView *treeview;
 	GtkTreeIter iter;
 	GtkTreeModel *model;
+	GtkTreeViewColumn *column;
 	GtkWidget *widget;
 	GtkTreePath *path;
 	gboolean selected;
+	gboolean scroll;
 
 	pk_client_get_role (client, &role, NULL, NULL);
 	egg_debug ("role = %s, package = %s:%s:%s", pk_role_enum_to_text (role),
@@ -581,6 +594,12 @@ gpk_update_viewer_package_cb (PkClient *client, const PkPackageObj *obj, gpointe
 	/* convert to string */
 	package_id = pk_package_id_to_string (obj->id);
 
+	/* are we simulating to get deps? */
+	if (role == PK_ROLE_ENUM_GET_DEPENDS) {
+		egg_debug ("ignoring %s as we are in the depends phase", package_id);
+		goto out;
+	}
+
 	/* used for progress */
 	if (!gpk_update_viewer_is_update_info (obj->info)) {
 		pk_package_id_free (package_id_last);
@@ -599,6 +618,13 @@ gpk_update_viewer_package_cb (PkClient *client, const PkPackageObj *obj, gpointe
 
 		gtk_tree_model_get_iter (model, &iter, path);
 
+		/* scroll to the active cell */
+		scroll = gconf_client_get_bool (gconf_client, GPK_CONF_UPDATE_VIEWER_SCROLL_ACTIVE, NULL);
+		if (scroll) {
+			column = gtk_tree_view_get_column (treeview, 3);
+			gtk_tree_view_scroll_to_cell (treeview, path, column, FALSE, 0.0f, 0.0f);
+		}
+
 		/* if the info is finished, change the status to past tense */
 		if (obj->info == PK_INFO_ENUM_FINISHED) {
 			gtk_tree_model_get (model, &iter,
@@ -612,10 +638,6 @@ gpk_update_viewer_package_cb (PkClient *client, const PkPackageObj *obj, gpointe
 		gtk_list_store_set (list_store_updates, &iter,
 				    GPK_UPDATES_COLUMN_STATUS, info, -1);
 
-		/* set size zero when we get a info update */
-		gtk_list_store_set (list_store_updates, &iter,
-				    GPK_UPDATES_COLUMN_SIZE, 0, -1);
-
 		gtk_tree_path_free (path);
 
 		/* set package description */
@@ -639,6 +661,7 @@ gpk_update_viewer_package_cb (PkClient *client, const PkPackageObj *obj, gpointe
 			    GPK_UPDATES_COLUMN_RESTART, PK_RESTART_ENUM_NONE,
 			    GPK_UPDATES_COLUMN_STATUS, PK_INFO_ENUM_UNKNOWN,
 			    GPK_UPDATES_COLUMN_SIZE, 0,
+			    GPK_UPDATES_COLUMN_SIZE_DISPLAY, 0,
 			    GPK_UPDATES_COLUMN_PERCENTAGE, 0,
 			    GPK_UPDATES_COLUMN_PULSE, -1,
 			    -1);
@@ -816,14 +839,23 @@ gpk_update_viewer_reconsider_info (GtkTreeModel *model)
 	if (number_total == 0) {
 		gtk_label_set_label (GTK_LABEL (widget), "");
 	} else {
-		text_size = g_format_size_for_display (size_total);
-		/* TRANSLATORS: how many updates are selected in the UI */
-		text = g_strdup_printf (ngettext ("%i update selected (%s)",
-						  "%i updates selected (%s)",
-						  number_total), number_total, text_size);
-		gtk_label_set_label (GTK_LABEL (widget), text);
-		g_free (text);
-		g_free (text_size);
+		if (size_total == 0) {
+			/* TRANSLATORS: how many updates are selected in the UI */
+			text = g_strdup_printf (ngettext ("%i update selected",
+							  "%i updates selected",
+							  number_total), number_total);
+			gtk_label_set_label (GTK_LABEL (widget), text);
+			g_free (text);
+		} else {
+			text_size = g_format_size_for_display (size_total);
+			/* TRANSLATORS: how many updates are selected in the UI, and the size of packages to download */
+			text = g_strdup_printf (ngettext ("%i update selected (%s)",
+							  "%i updates selected (%s)",
+							  number_total), number_total, text_size);
+			gtk_label_set_label (GTK_LABEL (widget), text);
+			g_free (text);
+			g_free (text_size);
+		}
 	}
 
 	widget = GTK_WIDGET (gtk_builder_get_object (builder, "label_summary"));
@@ -839,6 +871,7 @@ static void
 gpk_update_viewer_status_changed_cb (PkClient *client, PkStatusEnum status, gpointer data)
 {
 	GtkWidget *widget;
+	GdkWindow *window;
 	const gchar *text;
 	GdkDisplay *display;
 	GdkCursor *cursor;
@@ -852,12 +885,13 @@ gpk_update_viewer_status_changed_cb (PkClient *client, PkStatusEnum status, gpoi
 	gtk_widget_hide (widget);
 
 	/* set cursor back to normal */
+	window = gtk_widget_get_window (widget);
 	if (status == PK_STATUS_ENUM_FINISHED) {
-		gdk_window_set_cursor (widget->window, NULL);
+		gdk_window_set_cursor (window, NULL);
 	} else {
 		display = gdk_display_get_default ();
 		cursor = gdk_cursor_new_for_display (display, GDK_WATCH);
-		gdk_window_set_cursor (widget->window, cursor);
+		gdk_window_set_cursor (window, cursor);
 		gdk_cursor_unref (cursor);
 	}
 
@@ -1014,8 +1048,6 @@ out:
 	return ret;
 }
 
-
-
 /**
  * gpk_update_viewer_treeview_add_columns_update:
  **/
@@ -1046,7 +1078,7 @@ gpk_update_viewer_treeview_add_columns_update (GtkTreeView *treeview)
 	renderer = gpk_cell_renderer_info_new ();
 	g_object_set (renderer,
 		      "stock-size", GTK_ICON_SIZE_BUTTON,
-		      "ignore-values", "unknown,normal", NULL);
+		      "ignore-values", "unknown", NULL);
 	gtk_tree_view_column_pack_start (column, renderer, TRUE);
 	gtk_tree_view_column_add_attribute (column, renderer, "value", GPK_UPDATES_COLUMN_INFO);
 
@@ -1091,17 +1123,6 @@ gpk_update_viewer_treeview_add_columns_update (GtkTreeView *treeview)
 	gtk_tree_view_column_pack_start (column, renderer, TRUE);
 	gtk_tree_view_column_add_attribute (column, renderer, "value", GPK_UPDATES_COLUMN_STATUS);
 
-	/* size */
-	renderer = gpk_cell_renderer_size_new ();
-	g_object_set (renderer,
-		      "alignment", PANGO_ALIGN_RIGHT,
-		      "xalign", 1.0f,
-		      NULL);
-	g_object_set (renderer,
-		      "value", GPK_UPDATES_COLUMN_SIZE, NULL);
-	gtk_tree_view_column_pack_start (column, renderer, FALSE);
-	gtk_tree_view_column_add_attribute (column, renderer, "value", GPK_UPDATES_COLUMN_SIZE);
-
 	/* column for progress */
 	renderer = gpk_cell_renderer_spinner_new ();
 	g_object_set (renderer, "size", GTK_ICON_SIZE_BUTTON, NULL);
@@ -1115,6 +1136,27 @@ gpk_update_viewer_treeview_add_columns_update (GtkTreeView *treeview)
 	/* tooltips */
 	g_signal_connect (treeview, "query-tooltip", G_CALLBACK (gpk_update_viewer_treeview_query_tooltip_cb), NULL);
 	g_object_set (treeview, "has-tooltip", TRUE, NULL);
+
+	/* --- column for size --- */
+	column = gtk_tree_view_column_new ();
+	/* TRANSLATORS: a column that has size of each package */
+	gtk_tree_view_column_set_title (column, _("Size"));
+	gtk_tree_view_column_set_expand (GTK_TREE_VIEW_COLUMN (column), FALSE);
+	gtk_tree_view_column_set_sort_column_id (column, GPK_UPDATES_COLUMN_SIZE_DISPLAY);
+
+	/* size */
+	renderer = gpk_cell_renderer_size_new ();
+	g_object_set (renderer,
+		      "alignment", PANGO_ALIGN_RIGHT,
+		      "xalign", 1.0f,
+		      NULL);
+	g_object_set (renderer,
+		      "value", GPK_UPDATES_COLUMN_SIZE_DISPLAY, NULL);
+	gtk_tree_view_column_pack_start (column, renderer, FALSE);
+	gtk_tree_view_column_add_attribute (column, renderer, "value", GPK_UPDATES_COLUMN_SIZE_DISPLAY);
+
+	gtk_tree_view_append_column (treeview, column);
+	g_object_set_data (G_OBJECT (column), "tooltip-id", GINT_TO_POINTER (GPK_UPDATES_COLUMN_SIZE_DISPLAY));
 }
 
 /**
@@ -1527,11 +1569,28 @@ gpk_update_viewer_check_restart (PkRestartEnum restart)
 		message = _("Some of the updates that were installed require the computer to be restarted before the changes will be applied.");
 		/* TRANSLATORS: the button text for the restart */
 		button = _("Restart Computer");
-	} else {
+
+	} else if (restart == PK_RESTART_ENUM_SECURITY_SYSTEM) {
+		/* TRANSLATORS: the message text for the restart */
+		message = _("Some of the updates that were installed require the computer to be restarted to remain secure.");
+		/* TRANSLATORS: the button text for the restart */
+		button = _("Restart Computer");
+
+	} else if (restart == PK_RESTART_ENUM_SESSION) {
 		/* TRANSLATORS: the message text for the logout */
 		message = _("Some of the updates that were installed require you to log out and back in before the changes will be applied.");
 		/* TRANSLATORS: the button text for the logout */
 		button = _("Log Out");
+
+	} else if (restart == PK_RESTART_ENUM_SECURITY_SESSION) {
+		/* TRANSLATORS: the message text for the logout */
+		message = _("Some of the updates that were installed require you to log out and back in to remain secure.");
+		/* TRANSLATORS: the button text for the logout */
+		button = _("Log Out");
+
+	} else {
+		egg_warning ("unknown restart enum");
+		goto out;
 	}
 
 	/* show modal dialog */
@@ -1565,6 +1624,114 @@ out:
 	return ret;
 }
 
+/**
+ * gpk_update_viewer_deps_update_event_cb:
+ **/
+static void
+gpk_update_viewer_deps_update_event_cb (GpkHelperDepsUpdate *helper, GtkResponseType type, PkPackageList *deps_list, gpointer data)
+{
+	gboolean ret;
+	GError *error = NULL;
+	GtkTreeView *treeview;
+	GtkTreeModel *model;
+	gboolean valid;
+	GtkTreeIter iter;
+	gchar *package_id;
+	gchar *package_id_temp;
+	guint len;
+	guint i;
+	gboolean found;
+	gchar *text;
+	PkPackageId *id;
+	const PkPackageObj *obj;
+
+	/* get model */
+	treeview = GTK_TREE_VIEW (gtk_builder_get_object (builder, "treeview_updates"));
+	model = gtk_tree_view_get_model (treeview);
+
+	if (type != GTK_RESPONSE_YES) {
+		/* clear selection */
+		gpk_update_viewer_reconsider_info (model);
+		gpk_update_viewer_undisable_packages ();
+		goto out;
+	}
+
+	/* need to select or add packages in deps_list */
+	len = PK_OBJ_LIST(deps_list)->len;
+	for (i=0; i<len; i++) {
+		obj = pk_package_list_get_obj (deps_list, i);
+		found = FALSE;
+
+		/* find it and select it */
+		valid = gtk_tree_model_get_iter_first (model, &iter);
+		while (valid && !found) {
+			gtk_tree_model_get (model, &iter, GPK_UPDATES_COLUMN_ID, &package_id_temp, -1);
+			id = pk_package_id_new_from_string (package_id_temp);
+
+			/* we found a match */
+			if (pk_package_id_equal (id, obj->id)) {
+				egg_debug ("selecting %s", id->name);
+				gtk_list_store_set (GTK_LIST_STORE (model), &iter,
+						    GPK_UPDATES_COLUMN_SELECT, TRUE, -1);
+				found = TRUE;
+			}
+
+			g_free (package_id_temp);
+			pk_package_id_free (id);
+			valid = gtk_tree_model_iter_next (model, &iter);
+		}
+
+		/* not found, so add */
+		if (!found) {
+			package_id = pk_package_id_to_string (obj->id);
+			text = gpk_package_id_format_twoline (obj->id, obj->summary);
+			egg_debug ("adding: id=%s, text=%s", package_id, text);
+			gtk_list_store_append (list_store_updates, &iter);
+			gtk_list_store_set (list_store_updates, &iter,
+					    GPK_UPDATES_COLUMN_TEXT, text,
+					    GPK_UPDATES_COLUMN_ID, package_id,
+					    GPK_UPDATES_COLUMN_INFO, obj->info,
+					    GPK_UPDATES_COLUMN_SELECT, TRUE,
+					    GPK_UPDATES_COLUMN_SENSITIVE, FALSE,
+					    GPK_UPDATES_COLUMN_CLICKABLE, FALSE,
+					    GPK_UPDATES_COLUMN_RESTART, PK_RESTART_ENUM_NONE,
+					    GPK_UPDATES_COLUMN_STATUS, PK_INFO_ENUM_UNKNOWN,
+					    GPK_UPDATES_COLUMN_SIZE, 0,
+					    GPK_UPDATES_COLUMN_SIZE_DISPLAY, 0,
+					    GPK_UPDATES_COLUMN_PERCENTAGE, 0,
+					    GPK_UPDATES_COLUMN_PULSE, -1,
+					    -1);
+			g_free (text);
+			g_free (package_id);
+		}
+	}
+
+	/* if there are no entries selected, deselect the button */
+	gpk_update_viewer_reconsider_info (model);
+
+	/* reset client */
+	ret = pk_client_reset (client_primary, &error);
+	if (!ret) {
+		egg_warning ("cannot reset client: %s", error->message);
+		g_error_free (error);
+		goto out;
+	}
+
+	/* actually install packages this time */
+#if PK_CHECK_VERSION(0,5,0)
+	ret = pk_client_update_packages (client_primary, TRUE, install_package_ids, &error);
+#else
+	ret = pk_client_update_packages (client_primary, install_package_ids, &error);
+#endif
+	if (!ret) {
+		egg_warning ("cannot install packages: %s", error->message);
+		g_error_free (error);
+		goto out;
+	}
+out:
+	return;
+}
+
 /**
  * gpk_update_viewer_finished_cb:
  **/
@@ -1596,6 +1763,15 @@ gpk_update_viewer_finished_cb (PkClient *client, PkExitEnum exit, guint runtime,
 	widget = GTK_WIDGET (gtk_builder_get_object (builder, "button_quit"));
 	gtk_widget_set_sensitive (widget, TRUE);
 
+	/* need to handle retry with only_trusted=FALSE */
+	if (client == client_primary &&
+	    exit == PK_EXIT_ENUM_NEED_UNTRUSTED) {
+		egg_debug ("need to handle untrusted");
+		pk_client_set_only_trusted (client, FALSE);
+		gpk_update_viewer_primary_requeue (NULL);
+		return;
+	}
+
 	/* if secondary, ignore */
 	if (client == client_primary &&
 	    (exit == PK_EXIT_ENUM_KEY_REQUIRED ||
@@ -1615,6 +1791,15 @@ gpk_update_viewer_finished_cb (PkClient *client, PkExitEnum exit, guint runtime,
 		return;
 	}
 
+	/* finished depends check, show any extras */
+	if (role == PK_ROLE_ENUM_GET_DEPENDS) {
+
+		/* show deps dialog */
+		list = pk_client_get_package_list (client);
+		gpk_helper_deps_update_show (helper_deps_update, list);
+		g_object_unref (list);
+	}
+
 	if (role == PK_ROLE_ENUM_GET_UPDATES) {
 		/* get the download sizes */
 		if (update_list != NULL)
@@ -1682,6 +1867,8 @@ gpk_update_viewer_finished_cb (PkClient *client, PkExitEnum exit, guint runtime,
 	    (role == PK_ROLE_ENUM_UPDATE_SYSTEM ||
 	     role == PK_ROLE_ENUM_UPDATE_PACKAGES)) {
 
+		/* TODO: use ca_gtk_context_get_for_screen to allow use of GDK_MULTIHEAD_SAFE */
+
 		/* play the sound, using sounds from the naming spec */
 		ca_context_play (ca_gtk_context_get (), 0,
 				 /* TODO: add a new sound to the spec */
@@ -1703,7 +1890,9 @@ gpk_update_viewer_finished_cb (PkClient *client, PkExitEnum exit, guint runtime,
 
 		/* check restart */
 		if (restart_update == PK_RESTART_ENUM_SYSTEM ||
-		    restart_update == PK_RESTART_ENUM_SESSION) {
+		    restart_update == PK_RESTART_ENUM_SESSION ||
+		    restart_update == PK_RESTART_ENUM_SECURITY_SESSION ||
+		    restart_update == PK_RESTART_ENUM_SECURITY_SYSTEM) {
 			gpk_update_viewer_check_restart (restart_update);
 			g_main_loop_quit (loop);
 		}
@@ -1764,6 +1953,9 @@ gpk_update_viewer_finished_cb (PkClient *client, PkExitEnum exit, guint runtime,
 static GSList *active_rows = NULL;
 static guint active_row_timeout = 0;
 
+/**
+ * gpk_update_viewer_compare_refs:
+ **/
 static gint
 gpk_update_viewer_compare_refs (GtkTreeRowReference *a, GtkTreeRowReference *b)
 {
@@ -1788,6 +1980,9 @@ gpk_update_viewer_compare_refs (GtkTreeRowReference *a, GtkTreeRowReference *b)
 	return res;
 }
 
+/**
+ * gpk_update_viewer_pulse_active_rows:
+ **/
 static gboolean
 gpk_update_viewer_pulse_active_rows (void)
 {
@@ -1813,6 +2008,9 @@ gpk_update_viewer_pulse_active_rows (void)
 	return TRUE;
 }
 
+/**
+ * gpk_update_viewer_add_active_row:
+ **/
 static void
 gpk_update_viewer_add_active_row (GtkTreeModel *model, GtkTreePath *path)
 {
@@ -1826,6 +2024,9 @@ gpk_update_viewer_add_active_row (GtkTreeModel *model, GtkTreePath *path)
 	active_rows = g_slist_prepend (active_rows, ref);
 }
 
+/**
+ * gpk_update_viewer_remove_active_row:
+ **/
 static void
 gpk_update_viewer_remove_active_row (GtkTreeModel *model, GtkTreePath *path)
 {
@@ -1856,7 +2057,7 @@ gpk_update_viewer_remove_active_row (GtkTreeModel *model, GtkTreePath *path)
  **/
 static void
 gpk_update_viewer_progress_changed_cb (PkClient *client, guint percentage, guint subpercentage,
-				guint elapsed, guint remaining, gpointer data)
+				       guint elapsed, guint remaining, gpointer data)
 {
 	GtkWidget *widget;
 	GtkTreeView *treeview;
@@ -1864,6 +2065,8 @@ gpk_update_viewer_progress_changed_cb (PkClient *client, guint percentage, guint
 	GtkTreeIter iter;
 	GtkTreePath *path;
 	guint oldval;
+	guint size;
+	guint size_display;
 
 	widget = GTK_WIDGET (gtk_builder_get_object (builder, "progressbar_progress"));
 	gtk_widget_show (widget);
@@ -1885,16 +2088,22 @@ gpk_update_viewer_progress_changed_cb (PkClient *client, guint percentage, guint
 	}
 
 	gtk_tree_model_get_iter (model, &iter, path);
-
-	gtk_tree_model_get (model, &iter, GPK_UPDATES_COLUMN_PERCENTAGE, &oldval, -1);
+	gtk_tree_model_get (model, &iter,
+			    GPK_UPDATES_COLUMN_PERCENTAGE, &oldval,
+			    GPK_UPDATES_COLUMN_SIZE, &size,
+			    -1);
 	if ((oldval > 0 && oldval < 100) != (subpercentage > 0 && subpercentage < 100)) {
 		if (oldval > 0 && oldval < 100)
 			gpk_update_viewer_remove_active_row (model, path);
 		else
 			gpk_update_viewer_add_active_row (model, path);
 	}
+
+	size_display = size - ((size * subpercentage) / 100);
 	gtk_list_store_set (list_store_updates, &iter,
-			    GPK_UPDATES_COLUMN_PERCENTAGE, subpercentage, -1);
+			    GPK_UPDATES_COLUMN_PERCENTAGE, subpercentage,
+			    GPK_UPDATES_COLUMN_SIZE_DISPLAY, size_display,
+			    -1);
 
 	gtk_tree_path_free (path);
 }
@@ -1915,8 +2124,8 @@ gpk_update_viewer_error_code_cb (PkClient *client, PkErrorCodeEnum code, const g
 	}
 
 	/* ignore the ones we can handle */
-	if (code == PK_ERROR_ENUM_GPG_FAILURE ||
-	    code == PK_ERROR_ENUM_NO_LICENSE_AGREEMENT) {
+	if (code == PK_ERROR_ENUM_NO_LICENSE_AGREEMENT ||
+	    pk_error_code_is_need_untrusted (code)) {
 		egg_debug ("error ignored as we're handling %s\n%s", pk_error_enum_to_text (code), details);
 		return;
 	}
@@ -2171,6 +2380,7 @@ gpk_update_viewer_get_new_update_list (void)
 	GError *error = NULL;
 	GtkWidget *widget;
 	gchar *text = NULL;
+	PkBitfield filter = PK_FILTER_ENUM_NONE;
 
 	/* clear all widgets */
 	gtk_list_store_clear (list_store_updates);
@@ -2189,8 +2399,15 @@ gpk_update_viewer_get_new_update_list (void)
 		goto out;
 	}
 
+	/* only show newest updates? */
+	ret = gconf_client_get_bool (gconf_client, GPK_CONF_UPDATE_VIEWER_ONLY_NEWEST, NULL);
+	if (ret) {
+		egg_warning ("only showing newest updates");
+		filter = pk_bitfield_from_enums (PK_FILTER_ENUM_NEWEST, -1);
+	}
+
 	/* get new list */
-	ret = pk_client_get_updates (client_primary, PK_FILTER_ENUM_NONE, &error);
+	ret = pk_client_get_updates (client_primary, filter, &error);
 	if (!ret) {
 		egg_warning ("Failed to get updates: %s", error->message);
 		g_error_free (error);
@@ -2479,10 +2696,12 @@ static gboolean
 gpk_update_viewer_textview_motion_notify_event (GtkWidget *text_view, GdkEventMotion *event)
 {
 	gint x, y;
+	GdkWindow *window;
 
 	gtk_text_view_window_to_buffer_coords (GTK_TEXT_VIEW (text_view), GTK_TEXT_WINDOW_WIDGET, event->x, event->y, &x, &y);
 	gpk_update_viewer_textview_set_cursor (GTK_TEXT_VIEW (text_view), x, y);
-	gdk_window_get_pointer (text_view->window, NULL, NULL, NULL);
+	window = gtk_widget_get_window (text_view);
+	gdk_window_get_pointer (window, NULL, NULL, NULL);
 	return FALSE;
 }
 
@@ -2496,8 +2715,10 @@ static gboolean
 gpk_update_viewer_textview_visibility_notify_event (GtkWidget *text_view, GdkEventVisibility *event)
 {
 	gint wx, wy, bx, by;
+	GdkWindow *window;
 
-	gdk_window_get_pointer (text_view->window, &wx, &wy, NULL);
+	window = gtk_widget_get_window (text_view);
+	gdk_window_get_pointer (window, &wx, &wy, NULL);
 	gtk_text_view_window_to_buffer_coords (GTK_TEXT_VIEW (text_view), GTK_TEXT_WINDOW_WIDGET, wx, wy, &bx, &by);
 	gpk_update_viewer_textview_set_cursor (GTK_TEXT_VIEW (text_view), bx, by);
 	return FALSE;
@@ -2699,10 +2920,15 @@ main (int argc, char *argv[])
 	g_signal_connect (helper_eula, "event", G_CALLBACK (gpk_update_viewer_eula_event_cb), NULL);
 	gpk_helper_eula_set_parent (helper_eula, GTK_WINDOW (main_window));
 
+	helper_deps_update = gpk_helper_deps_update_new ();
+	g_signal_connect (helper_deps_update, "event", G_CALLBACK (gpk_update_viewer_deps_update_event_cb), NULL);
+	gpk_helper_deps_update_set_parent (helper_deps_update, GTK_WINDOW (main_window));
+
 	/* create list stores */
 	list_store_updates = gtk_list_store_new (GPK_UPDATES_COLUMN_LAST, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT,
 						 G_TYPE_BOOLEAN, G_TYPE_BOOLEAN, G_TYPE_BOOLEAN,
-						 G_TYPE_UINT, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_POINTER, G_TYPE_POINTER, G_TYPE_INT);
+						 G_TYPE_UINT, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_UINT,
+						 G_TYPE_UINT, G_TYPE_POINTER, G_TYPE_POINTER, G_TYPE_INT);
 	text_buffer = gtk_text_buffer_new (NULL);
 	gtk_text_buffer_create_tag (text_buffer, "para",
 				    "pixels_above_lines", 5,
@@ -2828,8 +3054,10 @@ main (int argc, char *argv[])
 	if (update_list != NULL)
 		g_object_unref (update_list);
 
+	g_strfreev (install_package_ids);
 	g_object_unref (helper_eula);
 	g_object_unref (helper_repo_signature);
+	g_object_unref (helper_deps_update);
 	g_object_unref (list_store_updates);
 	g_object_unref (text_buffer);
 	pk_package_id_free (package_id_last);
