@@ -37,13 +37,13 @@
 #include <gtk/gtk.h>
 #include <gconf/gconf-client.h>
 #include <libnotify/notify.h>
-#include <polkit-gnome/polkit-gnome.h>
 #include <packagekit-glib/packagekit.h>
 
 #include "egg-debug.h"
 #include "egg-string.h"
 
 #include "gpk-common.h"
+#include "gpk-session.h"
 #include "gpk-error.h"
 #include "gpk-watch.h"
 #include "gpk-modal-dialog.h"
@@ -73,9 +73,9 @@ struct GpkWatchPrivate
 	PkTaskList		*tlist;
 	PkRestartEnum		 restart;
 	GConfClient		*gconf_client;
-	PolKitGnomeAction	*restart_action;
 	guint			 set_proxy_timeout;
 	gchar			*error_details;
+	gboolean		 hide_warning;
 };
 
 typedef struct {
@@ -118,6 +118,62 @@ gpk_watch_cached_message_free (GpkWatchCachedMessage *cached_message)
 	g_free (cached_message);
 }
 
+/**
+ * gpk_watch_get_restart_required_tooltip:
+ **/
+static gchar *
+gpk_watch_get_restart_required_tooltip (GpkWatch *watch)
+{
+	gchar *package_loc = NULL;
+	gchar **packages = NULL;
+	guint len;
+	const gchar *title;
+	gchar *message = NULL;
+	gchar *text = NULL;
+
+	/* nothing */
+	if (watch->priv->restart == PK_RESTART_ENUM_NONE)
+		goto out;
+
+	/* size */
+	len = watch->priv->restart_package_names->len;
+	if (len == 0)
+		goto out;
+
+	/* localised title */
+	title = gpk_restart_enum_to_localised_text (watch->priv->restart);
+
+	/* non-security require */
+	if (watch->priv->restart == PK_RESTART_ENUM_SESSION ||
+	    watch->priv->restart == PK_RESTART_ENUM_SYSTEM) {
+
+		/* get localised list */
+		packages = pk_ptr_array_to_strv (watch->priv->restart_package_names);
+		package_loc = gpk_strv_join_locale (packages);
+		if (package_loc != NULL) {
+			/* TRANSLATORS: a list of packages is shown that need to restarted */
+			message = g_strdup_printf (ngettext ("This is due to the %s package being updated.",
+							     "This is due to the following packages being updated: %s.", len), package_loc);
+		} else {
+			/* TRANSLATORS: over 5 packages require the system to be restarted, don't list them all here */
+			message = g_strdup_printf (ngettext ("This is because %i package has been updated.",
+							     "This is because %i packages have been updated.", len), len);
+		}
+
+		/* join */
+		text = g_strdup_printf ("%s\n%s", title, message);
+		goto out;
+	}
+
+	/* just use title, as security requires are not the package that are updated */
+	text = g_strdup (title);
+out:
+	g_strfreev (packages);
+	g_free (package_loc);
+	g_free (message);
+	return text;
+}
+
 /**
  * gpk_watch_refresh_tooltip:
  **/
@@ -131,8 +187,7 @@ gpk_watch_refresh_tooltip (GpkWatch *watch)
 	GString *status;
 	const gchar *trailer;
 	const gchar *localised_status;
-	gchar *package_loc;
-	gchar **packages;
+	gchar *text;
 
 	g_return_val_if_fail (GPK_IS_WATCH (watch), FALSE);
 
@@ -142,38 +197,19 @@ gpk_watch_refresh_tooltip (GpkWatch *watch)
 	if (length == 0) {
 
 		/* any restart required? */
-		if (watch->priv->restart != PK_RESTART_ENUM_NONE) {
-			g_string_append (status, gpk_restart_enum_to_localised_text (watch->priv->restart));
-			g_string_append_c (status, '\n');
-
-			len = watch->priv->restart_package_names->len;
-
-			if (len > 0) {
-				packages = pk_ptr_array_to_strv (watch->priv->restart_package_names);
-				package_loc = gpk_strv_join_locale (packages);
-				g_strfreev (packages);
-				if (package_loc != NULL) {
-					/* TRANSLATORS: a list of packages is shown that need to restarted */
-					g_string_append_printf (status, ngettext ("Package: %s", "Packages: %s", len), package_loc);
-				} else {
-					/* TRANSLATORS: over 5 packages require the system to be restarted, don't list them all here */
-					g_string_append_printf (status, ngettext ("%i package", "%i packages", len), len);
-				}
-				g_string_append_c (status, '\n');
-				g_free (package_loc);
-			}
-
-			/* remove final \n */
-			if (status->len > 0)
-				g_string_set_size (status, status->len - 1);
-
-			goto out;
+		text = gpk_watch_get_restart_required_tooltip (watch);
+		if (text != NULL) {
+			g_string_append (status, text);
 		}
+		g_free (text);
 
 		/* do we have any cached messages to show? */
 		len = watch->priv->cached_messages->len;
 		if (len > 0) {
-			g_string_append_printf (status, ngettext ("%i message from the package manager", "%i messages from the package manager", len), len);
+			if (status->len > 0)
+				g_string_append_c (status, '\n');
+			g_string_append_printf (status, ngettext ("%i message from the package manager",
+								  "%i messages from the package manager", len), len);
 			goto out;
 		}
 
@@ -227,6 +263,7 @@ gpk_watch_task_list_to_status_bitfield (GpkWatch *watch)
 {
 	gboolean ret;
 	gboolean active;
+	gboolean watch_active;
 	guint i;
 	guint length;
 	PkBitfield status = 0;
@@ -239,6 +276,9 @@ gpk_watch_task_list_to_status_bitfield (GpkWatch *watch)
 	if (length == 0)
 		goto out;
 
+	/* do we watch active transactions */
+	watch_active = gconf_client_get_bool (watch->priv->gconf_client, GPK_CONF_WATCH_ACTIVE_TRANSACTIONS, NULL);
+
 	/* add each status to a list */
 	for (i=0; i<length; i++) {
 		item = pk_task_list_get_item (watch->priv->tlist, i);
@@ -256,7 +296,7 @@ gpk_watch_task_list_to_status_bitfield (GpkWatch *watch)
 
 		/* add to bitfield calculation */
 		egg_debug ("%s %s (active:%i)", item->tid, pk_status_enum_to_text (item->status), active);
-		if (!active)
+		if (!active || watch_active)
 			pk_bitfield_add (status, item->status);
 	}
 out:
@@ -324,15 +364,16 @@ gpk_watch_refresh_icon (GpkWatch *watch)
 	}
 
 	/* any restart required? */
-	if (watch->priv->restart != PK_RESTART_ENUM_NONE) {
-		icon_name = gpk_restart_enum_to_dialog_icon_name (watch->priv->restart);
+	if (watch->priv->restart != PK_RESTART_ENUM_NONE &&
+	    watch->priv->hide_warning == FALSE) {
+		icon_name = gpk_restart_enum_to_icon_name (watch->priv->restart);
 		goto out;
 	}
 
 	/* do we have any cached messages to show? */
 	len = watch->priv->cached_messages->len;
 	if (len > 0) {
-		icon_name = "emblem-important";
+		icon_name = "pk-setup";
 		goto out;
 	}
 
@@ -367,11 +408,11 @@ gpk_watch_libnotify_cb (NotifyNotification *notification, gchar *action, gpointe
 {
 	GpkWatch *watch = GPK_WATCH (data);
 
-	if (egg_strequal (action, "do-not-show-notify-complete")) {
+	if (g_strcmp0 (action, "do-not-show-notify-complete") == 0) {
 		egg_debug ("set %s to FALSE", GPK_CONF_NOTIFY_COMPLETED);
 		gconf_client_set_bool (watch->priv->gconf_client, GPK_CONF_NOTIFY_COMPLETED, FALSE, NULL);
 
-	} else if (egg_strequal (action, "show-error-details")) {
+	} else if (g_strcmp0 (action, "show-error-details") == 0) {
 		/* TRANSLATORS: The detailed error if the user clicks "more info" */
 		gpk_error_dialog (_("Error details"), _("Package manager error details"), watch->priv->error_details);
 
@@ -395,8 +436,13 @@ gpk_watch_task_list_finished_cb (PkTaskList *tlist, PkClient *client, PkExitEnum
 	gchar *text = NULL;
 	gchar *message = NULL;
 	NotifyNotification *notification;
+#if PK_CHECK_VERSION(0,5,0)
+	const PkRequireRestartObj *obj;
+	PkObjList *array;
+#else
 	PkPackageId *id;
-	const GPtrArray	*array;
+	const GPtrArray *array;
+#endif
 
 	g_return_if_fail (GPK_IS_WATCH (watch));
 
@@ -408,6 +454,12 @@ gpk_watch_task_list_finished_cb (PkTaskList *tlist, PkClient *client, PkExitEnum
 	}
 	egg_debug ("role=%s, text=%s", pk_role_enum_to_text (role), text);
 
+	/* is it worth processing? */
+	if (exit_enum != PK_EXIT_ENUM_SUCCESS) {
+		egg_debug ("not processing, as didn't complete okay");
+		goto out;
+	}
+
 	/* show an icon if the user needs to reboot */
 	if (role == PK_ROLE_ENUM_UPDATE_PACKAGES ||
 	    role == PK_ROLE_ENUM_INSTALL_PACKAGES ||
@@ -415,14 +467,28 @@ gpk_watch_task_list_finished_cb (PkTaskList *tlist, PkClient *client, PkExitEnum
 		/* if more important than what we are already showing, then update the icon */
 		restart = pk_client_get_require_restart (client);
 		if (restart > watch->priv->restart) {
-
+#if PK_CHECK_VERSION(0,5,0)
+			/* list packages requiring this */
+			array = pk_client_get_require_restart_list (client);
+			if (array == NULL) {
+				egg_warning ("no data about restarts, perhaps not buffered");
+				goto no_data;
+			}
+			for (i=0; i<array->len; i++) {
+				obj = pk_obj_list_index (array, i);
+				if (obj->restart >= restart)
+					g_ptr_array_add (watch->priv->restart_package_names, g_strdup (obj->id->name));
+			}
+			g_object_unref (array);
+no_data:
+#else
 			/* list packages requiring this */
 			array = pk_client_get_require_restart_list (client);
 			for (i=0; i<array->len; i++) {
 				id = g_ptr_array_index (array, i);
 				g_ptr_array_add (watch->priv->restart_package_names, g_strdup (id->name));
 			}
-
+#endif
 			/* save new restart */
 			watch->priv->restart = restart;
 		}
@@ -434,12 +500,6 @@ gpk_watch_task_list_finished_cb (PkTaskList *tlist, PkClient *client, PkExitEnum
 		goto out;
 	}
 
-	/* is it worth showing a UI? */
-	if (exit_enum != PK_EXIT_ENUM_SUCCESS) {
-		egg_debug ("not notifying, as didn't complete okay");
-		goto out;
-	}
-
 	/* are we accepting notifications */
 	value = gconf_client_get_bool (watch->priv->gconf_client, GPK_CONF_NOTIFY_COMPLETED, NULL);
 	if (!value) {
@@ -668,7 +728,6 @@ gpk_watch_about_dialog_url_cb (GtkAboutDialog *about, const char *address, gpoin
 {
 	GError *error = NULL;
 	gboolean ret;
-	char *cmdline;
 	GdkScreen *gscreen;
 	gchar *url;
 	gchar *protocol = (gchar*) data;
@@ -680,19 +739,7 @@ gpk_watch_about_dialog_url_cb (GtkAboutDialog *about, const char *address, gpoin
 
 	gscreen = gtk_window_get_screen (GTK_WINDOW (about));
 
-	cmdline = g_strconcat ("xdg-open ", url, NULL);
-	ret = gdk_spawn_command_line_on_screen (gscreen, cmdline, &error);
-	g_free (cmdline);
-
-	if (ret)
-		goto out;
-
-	g_error_free (error);
-	error = NULL;
-
-	cmdline = g_strconcat ("gnome-open ", url, NULL);
-	ret = gdk_spawn_command_line_on_screen (gscreen, cmdline, &error);
-	g_free (cmdline);
+	ret = gtk_show_uri (gscreen, url, gtk_get_current_event_time (), &error);
 
 	if (!ret) {
 		/* TRANSLATORS: We couldn't launch the tool, normally a packaging problem */
@@ -700,7 +747,6 @@ gpk_watch_about_dialog_url_cb (GtkAboutDialog *about, const char *address, gpoin
 		g_error_free (error);
 	}
 
-out:
 	g_free (url);
 }
 
@@ -792,15 +838,6 @@ gpk_watch_popup_menu_cb (GtkStatusIcon *status_icon, guint button, guint32 times
 		gtk_menu_shell_select_first (GTK_MENU_SHELL (menu), FALSE);
 }
 
-/**
- * gpk_watch_restart_cb:
- **/
-static void
-gpk_watch_restart_cb (PolKitGnomeAction *action, gpointer data)
-{
-	gpk_restart_system ();
-}
-
 /**
  * gpk_watch_menu_show_messages_cb:
  **/
@@ -1212,7 +1249,8 @@ gpk_watch_menu_hide_restart_cb (GtkMenuItem *item, gpointer data)
 	g_return_if_fail (GPK_IS_WATCH (watch));
 
 	/* hide */
-	gtk_status_icon_set_visible (watch->priv->status_icon, FALSE);
+	watch->priv->hide_warning = TRUE;
+	gpk_watch_refresh_icon (watch);
 }
 
 /**
@@ -1222,8 +1260,22 @@ static void
 gpk_watch_menu_log_out_cb (GtkMenuItem *item, gpointer data)
 {
 	GpkWatch *watch = GPK_WATCH (data);
+	GpkSession *session;
 	g_return_if_fail (GPK_IS_WATCH (watch));
-	gpk_session_logout ();
+
+	/* just ask for logout */
+	session = gpk_session_new ();
+	gpk_session_logout (session);
+	g_object_unref (session);
+}
+
+/**
+ * gpk_watch_menu_restart_cb:
+ **/
+static void
+gpk_watch_menu_restart_cb (GtkMenuItem *item, gpointer data)
+{
+	gpk_restart_system ();
 }
 
 /**
@@ -1262,7 +1314,9 @@ gpk_watch_activate_status_cb (GtkStatusIcon *status_icon, GpkWatch *watch)
 	}
 
 	/* log out session */
-	if (watch->priv->restart == PK_RESTART_ENUM_SESSION) {
+	if (watch->priv->restart == PK_RESTART_ENUM_SESSION ||
+	    watch->priv->restart == PK_RESTART_ENUM_SECURITY_SESSION) {
+		/* TRANSLATORS: log out of the session */
 		widget = gtk_image_menu_item_new_with_mnemonic (_("_Log out"));
 		image = gtk_image_new_from_icon_name ("system-log-out", GTK_ICON_SIZE_MENU);
 		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (widget), image);
@@ -1273,8 +1327,14 @@ gpk_watch_activate_status_cb (GtkStatusIcon *status_icon, GpkWatch *watch)
 	}
 
 	/* restart computer */
-	if (watch->priv->restart == PK_RESTART_ENUM_SYSTEM) {
-		widget = gtk_action_create_menu_item (GTK_ACTION (watch->priv->restart_action));
+	if (watch->priv->restart == PK_RESTART_ENUM_SYSTEM ||
+	    watch->priv->restart == PK_RESTART_ENUM_SECURITY_SYSTEM) {
+		/* TRANSLATORS: this menu item restarts the computer after an update */
+		widget = gtk_image_menu_item_new_with_mnemonic (_("_Restart computer"));
+		image = gtk_image_new_from_icon_name ("system-shutdown", GTK_ICON_SIZE_MENU);
+		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (widget), image);
+		g_signal_connect (G_OBJECT (widget), "activate",
+				  G_CALLBACK (gpk_watch_menu_restart_cb), watch);
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), widget);
 		show_hide = TRUE;
 	}
@@ -1297,22 +1357,6 @@ gpk_watch_activate_status_cb (GtkStatusIcon *status_icon, GpkWatch *watch)
 			1, gtk_get_current_event_time());
 }
 
-/**
- * pk_connection_changed_cb:
- **/
-static void
-pk_connection_changed_cb (PkConnection *pconnection, gboolean connected, GpkWatch *watch)
-{
-	g_return_if_fail (GPK_IS_WATCH (watch));
-	egg_debug ("connected=%i", connected);
-	if (connected) {
-		gpk_watch_refresh_icon (watch);
-		gpk_watch_refresh_tooltip (watch);
-	} else {
-		gtk_status_icon_set_visible (watch->priv->status_icon, FALSE);
-	}
-}
-
 /**
  * gpk_watch_locked_cb:
  **/
@@ -1344,7 +1388,7 @@ gpk_watch_get_proxy_ftp (GpkWatch *watch)
 
 	/* common case, a direct connection */
 	mode = gconf_client_get_string (watch->priv->gconf_client, "/system/proxy/mode", NULL);
-	if (egg_strequal (mode, "none")) {
+	if (g_strcmp0 (mode, "none") == 0) {
 		egg_debug ("not using session proxy");
 		goto out;
 	}
@@ -1386,7 +1430,7 @@ gpk_watch_get_proxy_http (GpkWatch *watch)
 
 	/* common case, a direct connection */
 	mode = gconf_client_get_string (watch->priv->gconf_client, "/system/proxy/mode", NULL);
-	if (egg_strequal (mode, "none")) {
+	if (g_strcmp0 (mode, "none") == 0) {
 		egg_debug ("not using session proxy");
 		goto out;
 	}
@@ -1560,6 +1604,23 @@ gpk_watch_button_cancel_cb (GtkWidget *widget, GpkWatch *watch)
 	}
 }
 
+/**
+ * gpk_watch_connection_changed_cb:
+ **/
+static void
+gpk_watch_connection_changed_cb (PkConnection *pconnection, gboolean connected, GpkWatch *watch)
+{
+	g_return_if_fail (GPK_IS_WATCH (watch));
+	egg_debug ("connected=%i", connected);
+	if (connected) {
+		gpk_watch_refresh_icon (watch);
+		gpk_watch_refresh_tooltip (watch);
+		gpk_watch_set_proxies (watch);
+	} else {
+		gtk_status_icon_set_visible (watch->priv->status_icon, FALSE);
+	}
+}
+
 /**
  * gpk_watch_init:
  * @watch: This class instance
@@ -1567,13 +1628,11 @@ gpk_watch_button_cancel_cb (GtkWidget *widget, GpkWatch *watch)
 static void
 gpk_watch_init (GpkWatch *watch)
 {
-	PolKitAction *pk_action;
-	PolKitGnomeAction *restart_action;
-
 	watch->priv = GPK_WATCH_GET_PRIVATE (watch);
 	watch->priv->error_details = NULL;
 	watch->priv->notification_cached_messages = NULL;
 	watch->priv->restart = PK_RESTART_ENUM_NONE;
+	watch->priv->hide_warning = FALSE;
 
 	watch->priv->gconf_client = gconf_client_get_default ();
 
@@ -1629,26 +1688,9 @@ gpk_watch_init (GpkWatch *watch)
 
 	watch->priv->pconnection = pk_connection_new ();
 	g_signal_connect (watch->priv->pconnection, "connection-changed",
-			  G_CALLBACK (pk_connection_changed_cb), watch);
+			  G_CALLBACK (gpk_watch_connection_changed_cb), watch);
 	if (pk_connection_valid (watch->priv->pconnection))
-		pk_connection_changed_cb (watch->priv->pconnection, TRUE, watch);
-
-	pk_action = polkit_action_new ();
-	polkit_action_set_action_id (pk_action, "org.freedesktop.consolekit.system.restart");
-
-	restart_action = polkit_gnome_action_new_default ("restart-system", pk_action,
-							  /* TRANSLATORS: This button restarts the computer after an update */
-							  _("_Restart computer"), NULL);
-	g_object_set (restart_action,
-		      "no-icon-name", "system-shutdown",
-		      "auth-icon-name", "system-shutdown",
-		      "yes-icon-name","system-shutdown",
-		      "self-blocked-icon-name", "system-shutdown",
-		      NULL);
-	polkit_action_unref (pk_action);
-	g_signal_connect (restart_action, "activate",
-			  G_CALLBACK (gpk_watch_restart_cb), NULL);
-	watch->priv->restart_action = restart_action;
+		gpk_watch_connection_changed_cb (watch->priv->pconnection, TRUE, watch);
 
 	/* watch proxy keys */
 	gconf_client_add_dir (watch->priv->gconf_client, GPK_WATCH_GCONF_PROXY_HTTP,
@@ -1698,7 +1740,6 @@ gpk_watch_finalize (GObject *object)
 	g_object_unref (watch->priv->control);
 	g_object_unref (watch->priv->pconnection);
 	g_object_unref (watch->priv->gconf_client);
-	g_object_unref (watch->priv->restart_action);
 	g_object_unref (watch->priv->client_primary);
 	g_object_unref (watch->priv->dialog);
 
